﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using NUnit.Framework;
using Library.Model;
using Library.Model.Loans;
using Library.Model.Users;
using Library.Model.Items;
using Library.Model.ItemCopies;
using Library.Repository.EF;

namespace Library.IntegrationTests
{
	[TestFixture]
	public class EFLoanRepositoryTests
	{
		[Test]
		public void Can_Add_Loan_To_Repository()
		{
			// Add a User
			User user = CreateUser();
			// Add a ItemCopy
			ItemCopy copy = CreateItemCopy();

			// Create a Loan
			UserRepository userRepository = new UserRepository();
			ItemCopyRepository itemCopyRepository = new ItemCopyRepository();

			Loan loan = new Loan();
			loan.User = userRepository.FindBy(1);
			loan.ItemCopy = itemCopyRepository.FindBy(1);
			loan.LoanStatus = Model.Loans.LoanStatus.Unprocessed;
			loan.LoanMethod = Model.Loans.LoanMethod.TakenOut;
			loan.LoanPeriod = 2;

			LoanRepository loanRepository = new LoanRepository();
			loanRepository.Add(loan);
			Assert.AreEqual(1, loanRepository.FindAll().Count);

		}

		#region Helper Methods

		private User CreateUser()
		{
			UserRepository userRepository = new UserRepository();

			User user = new User();
			user.FirstName = "Jhon";
			user.LastName = "Smith";
			user.EmailAddress = "jhon@microsoft.com";
			user.Status = Model.Users.Status.Active;
			user.ContactAddress = new Address("USA", "Vegas", "Walls Fare", "13068");

			userRepository.Add(user);
			return user;
		}

		private ItemCopy CreateItemCopy()
		{
			ItemCopyRepository itemCopyRepository = new ItemCopyRepository();

			ItemCopy copy = new ItemCopy();
			copy.Barcode = "SAXA45";
			copy.Condition = "Used";
			copy.DateOfRecord = DateTime.Now;
			copy.IsAvailable = true;
			copy.IsReferenceUseOnly = true;
			copy.Item = new Book() {
				Title = "The C# Programming Language",
				Description = "A Must-Read",
				ISBN = "10328554895",
				PublicationDate = DateTime.Now
			};

			itemCopyRepository.Add(copy);
			return copy;
		}

		#endregion
	}
}
