﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Library.Model
{
	public class Address
	{
		private string _city;
		private string _town;
		private string _street;
		private string _postalCode;

		public Address()
		{ }

		public Address(string city, string town, string street, string postalCode)
		{
			_city = city;
			_town = town;
			_street = street;
			_postalCode = postalCode;

			if (GetBrokenRules().Count > 0) {
				throw new Exception(String.Format("The address is trying to be created with invalid data.{0}", GetBrokenRulesToString()));
			}
		}

		public string City
		{
			get { return _city; }
			set { _city = value; }
		}

		public string Town
		{
			get { return _town; }
			set { _town = value; }
		}

		public string Street
		{
			get { return _street; }
			set { _street = value; }
		}

		public string PostalCode
		{
			get { return _postalCode; }
			set { _postalCode = value; }
		}

		public List<BrokenBusinessRule> GetBrokenRules()
		{
			List<BrokenBusinessRule> brokenRules = new List<BrokenBusinessRule>();

			if (string.IsNullOrEmpty(City))
				brokenRules.Add(new BrokenBusinessRule("City", "A city must be specified for an address"));
			if (string.IsNullOrEmpty(Town))
				brokenRules.Add(new BrokenBusinessRule("Town", "A town must be specified for an address"));
			if (string.IsNullOrEmpty(Street))
				brokenRules.Add(new BrokenBusinessRule("Street", "A street must be specified for an address"));
			if (string.IsNullOrEmpty(PostalCode))
				brokenRules.Add(new BrokenBusinessRule("PostalCode", "A postal code must be specified for an address"));
			return brokenRules;
		}

		public string GetBrokenRulesToString()
		{
			StringBuilder sbBrokenRules = new StringBuilder();

			foreach (BrokenBusinessRule br in GetBrokenRules()) {
				sbBrokenRules.Append(br.Rule);
			}
			return sbBrokenRules.ToString();
		}
	}
}
