﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Library.Model.Users;
using Library.Model.Items;
using Library.Model.ItemCopies;
using Library.Model.Reservations;

namespace Library.Model.Loans
{
	public class Loan
	{
		private int _loanId;
		private DateTime? _dateIssued;
		private DateTime? _dateDueForReturn;
		private int? _loanPeriod;
		private DateTime? _dateReturned;
		private LoanMethod _loanMethod;
		private LoanStatus _status;
		private Reservation _reservation;
		private User _user;
		private ItemCopy _itemCopy;

		public int LoanId
		{
			get { return _loanId; }
			set { _loanId = value; }
		}

		public DateTime? DateIssued
		{
			get { return _dateIssued; }
			set { _dateIssued = value; }
		}

		public DateTime? DateReturned
		{
			get { return _dateReturned; }
			set { _dateReturned = value; }
		}

		public DateTime? DateDueForReturn
		{
			get { return _dateDueForReturn; }
			set { _dateDueForReturn = value; }
		}

		public LoanMethod LoanMethod
		{
			get { return _loanMethod; }
			set { _loanMethod = value; }
		}

		public LoanStatus LoanStatus
		{
			get { return _status; }
			set { _status = value; }
		}

		public virtual Reservation Reservation
		{
			get { return _reservation; }
			set { _reservation = value; }
		}

		public int? LoanPeriod
		{
			get { return _loanPeriod; }
			set { _loanPeriod = value; }
		}

		public virtual User User
		{
			get { return _user; }
			set { _user = value; }
		}

		public virtual ItemCopy ItemCopy
		{
			get { return _itemCopy; }
			set { _itemCopy = value; }
		}


		public List<BrokenBusinessRule> GetBrokenRules()
		{
			List<BrokenBusinessRule> brokenRules = new List<BrokenBusinessRule>();

			if (LoanPeriod != null) {
				if (LoanPeriod <= 0) {
					brokenRules.Add(new BrokenBusinessRule("LoanPeriod", "A loan period must be greater than zero."));
				}
			}
			//if (DateIssued <= DateTime.Now)
			//	brokenRules.Add(new BrokenBusinessRule("DateIssued", "The Date Issued must be greater tha the current time."));
			//if (DateDueForReturn < DateIssued)
			//	brokenRules.Add(new BrokenBusinessRule("DateDueForReturn", "Invalid due date for return"));
			if (User == null) {
				brokenRules.Add(new BrokenBusinessRule("User", "A loan must have a user."));
			} else if (User.GetBrokenRules().Count > 0) {
				AddToBrokenRulesList(brokenRules, User.GetBrokenRules());
			}
			if (ItemCopy == null) {
				brokenRules.Add(new BrokenBusinessRule("Item", "A loan must have an item."));
			} else if (ItemCopy.GetBrokenRules().Count > 0) {
				AddToBrokenRulesList(brokenRules, ItemCopy.GetBrokenRules());
			}				
			return brokenRules;
		}

		private void AddToBrokenRulesList(List<BrokenBusinessRule> currentBrokenRules, List<BrokenBusinessRule> brokenRulesToAdd)
		{
			foreach (BrokenBusinessRule br in brokenRulesToAdd) {
				currentBrokenRules.Add(br);
			}
		}

		#region WorkFlow Methods

		public void Lend()
		{
			ThrowExceptionIfLoanIsInvalid("Lending");

			if (ItemCopy.IsAvailable && User.IsActive()) {
				LoanStatus = LoanStatus.Given;
				GenerateReturnDate();
				ItemCopy.IsAvailable = false;
			}
		}

		public void Return()
		{
			ThrowExceptionIfLoanIsInvalid("Returning");

			if (LoanStatus != LoanStatus.Returned) {
				if (LoanStatus == LoanStatus.Expired) {
					User.Status = Model.Users.Status.Frozen;
				}
				LoanStatus = LoanStatus.Returned;
				DateReturned = DateTime.Now;
				ItemCopy.IsAvailable = true;
			}
		}

		private void GenerateReturnDate()
		{
			DateIssued = DateTime.Now;
			if (this.LoanMethod == LoanMethod.TakenOut)
				DateDueForReturn = DateIssued.Value.AddDays(LoanPeriod.Value);			
		}

		public void ThrowExceptionIfLoanIsInvalid(string process)
		{
			if (GetBrokenRules().Count > 0) {
				StringBuilder sbBrokenRules = new StringBuilder();
				foreach (BrokenBusinessRule br in GetBrokenRules()) {
					sbBrokenRules.Append(br.Rule);
				}
				throw new InvalidLoanException(string.Format("The '{0}' process cannot be started because"
					+ "the loan is in an invalid state.{1}", process, sbBrokenRules.ToString()));
			}
		}

		#endregion

	}
}
