﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Library.Model.Loans
{
	public class LoanService
	{
		private ILoanRepository _loanRepository;

		public LoanService(ILoanRepository loanRepository)
		{
			_loanRepository = loanRepository;
		}

		public void ProcessLoan(Loan loan)
		{
			if (loan.GetBrokenRules().Count > 0)
				throw new InvalidLoanException(string.Format("The Loan cannot be processed. {0}", GetBrokenRulesToStringFor(loan.GetBrokenRules())));
			
			loan.Lend();
			if (loan.LoanStatus == LoanStatus.Given)
				_loanRepository.Save(loan);
		}

		public void ProcessReturn(Loan loan)
		{
			if (loan.GetBrokenRules().Count > 0)
				throw new InvalidLoanException(string.Format("The Return cannot be processed. {0}", GetBrokenRulesToStringFor(loan.GetBrokenRules())));

			loan.Return();
			_loanRepository.Save(loan);
		}

		private string GetBrokenRulesToStringFor(List<BrokenBusinessRule> brokenRules)
		{
			StringBuilder sbBrokenRules = new StringBuilder();

			foreach (BrokenBusinessRule br in brokenRules) {
				sbBrokenRules.Append(br.Rule);
			}
			return sbBrokenRules.ToString();
		}
	}
}
