﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Library.Model.Items;

namespace Library.Model.Reservations
{
	public class Reservation
	{
		private int _reservationId;
		private DateTime _reservationDate;
		private DateTime _expirationDate;
		private Item _item;

		public int ReservationId
		{
			get { return _reservationId; }
			set { _reservationId = value; }
		}

		public DateTime ReservationDate
		{
			get { return _reservationDate; }
		}

		public DateTime ExpirationDate
		{
			get { return _expirationDate; }
		}

		public virtual Item Item
		{
			get { return _item; }
			set { _item = value; }
		}

		public List<BrokenBusinessRule> GetBrokenRules()
		{
			List<BrokenBusinessRule> brokenRules = new List<BrokenBusinessRule>();

			if (ReservationDate < DateTime.Now) {
				brokenRules.Add(new BrokenBusinessRule("ReservationDate", "The Reservation Date must be equal or greater than the current date"));
			}
			if (ExpirationDate < ReservationDate) {
				brokenRules.Add(new BrokenBusinessRule("ReservationExpiryDate", "Invalid Expiry Date for the reservation"));
			}
			return brokenRules;
		}

		public string GetBrokenRulesToString()
		{
			StringBuilder sbBrokenRules = new StringBuilder();

			foreach (BrokenBusinessRule br in GetBrokenRules()) {
				sbBrokenRules.Append(br.Rule);				
			}
			return sbBrokenRules.ToString();
		}

		public bool HasExpired()
		{
			return ExpirationDate > DateTime.Now;
		}
	}
}
