﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Library.Model.Loans;
using Library.Model.SavedItems;

namespace Library.Model.Users
{
	public class User
	{
		private int _userId;
		private string _firstName;
		private string _lastName;
		private string _emailAddress;
		private Status _status;
		private Address _contactAddress;
		private IList<SavedItem> _savedItems;
		private IList<Loan> _loans;

		public User()
		{
			_loans = new List<Loan>();
		}

		public int UserId
		{
			get { return _userId; }
			set { _userId = value; }
		}

		public string FirstName
		{
			get { return _firstName; }
			set { _firstName = value; }
		}

		public string LastName
		{
			get { return _lastName; }
			set { _lastName = value; }
		}

		public string EmailAddress
		{
			get { return _emailAddress; }
			set { _emailAddress = value; }
		}

		public Status Status
		{
			get { return _status; }
			set { _status = value; }
		}

		public Address ContactAddress
		{
			get { return _contactAddress; }
			set { _contactAddress = value; }
		}

		public virtual IList<SavedItem> SavedItems
		{
			get { return _savedItems; }
			set { _savedItems = value; }
		}

		public virtual IList<Loan> Loans
		{
			get { return _loans; }
			set { _loans = value; }
		}

		public List<BrokenBusinessRule> GetBrokenRules()
		{
			List<BrokenBusinessRule> brokenRules = new List<BrokenBusinessRule>();

			if (string.IsNullOrEmpty(FirstName))
				brokenRules.Add(new BrokenBusinessRule("FirstName", "You must enter a first name for the user."));
			if (string .IsNullOrEmpty(LastName))
				brokenRules.Add(new BrokenBusinessRule("LastName", "You must enter a last name for the user."));
			if (string.IsNullOrEmpty(EmailAddress))
				brokenRules.Add(new BrokenBusinessRule("EmailAddress", "Your must enter a email address for the user"));
			if (ContactAddress == null) {
				brokenRules.Add(new BrokenBusinessRule("ContactAddress", "You must enter a contact address for the user"));
			} else if (ContactAddress.GetBrokenRules().Count > 0) {
				AddToBrokenRulesList(brokenRules, ContactAddress.GetBrokenRules());
			}
			return brokenRules;
		}

		private void AddToBrokenRulesList(List<BrokenBusinessRule> currentBrokenRules, List<BrokenBusinessRule> brokenRulesToAdd)
		{
			foreach (BrokenBusinessRule br in brokenRulesToAdd) {
				currentBrokenRules.Add(br);
			}
		}

		public bool IsActive()
		{
			return Status == Status.Active;
		}

	}
}
