namespace Library.Repository.EF.Migrations
{
    using System;
    using System.Data.Entity.Migrations;
    
    public partial class Initial : DbMigration
    {
        public override void Up()
        {
            CreateTable(
                "dbo.Categories",
                c => new
                    {
                        CategoryId = c.Int(nullable: false, identity: true),
                        Name = c.String(),
                        Description = c.String(),
                        ParentCategory_CategoryId = c.Int(),
                    })
                .PrimaryKey(t => t.CategoryId)
                .ForeignKey("dbo.Categories", t => t.ParentCategory_CategoryId)
                .Index(t => t.ParentCategory_CategoryId);
            
            CreateTable(
                "dbo.ItemCopies",
                c => new
                    {
                        ItemCopyId = c.Int(nullable: false, identity: true),
                        Barcode = c.String(),
                        Tag = c.String(),
                        Condition = c.String(),
                        IsReferenceUseOnly = c.Boolean(nullable: false),
                        IsAvailable = c.Boolean(nullable: false),
                        DateOfRecord = c.DateTime(nullable: false),
                        Item_ItemId = c.Int(),
                    })
                .PrimaryKey(t => t.ItemCopyId)
                .ForeignKey("dbo.Items", t => t.Item_ItemId)
                .Index(t => t.Item_ItemId);
            
            CreateTable(
                "dbo.SavedItems",
                c => new
                    {
                        ItemId = c.Int(nullable: false),
                        UserId = c.Int(nullable: false),
                        DateAdded = c.DateTime(nullable: false),
                    })
                .PrimaryKey(t => new { t.ItemId, t.UserId })
                .ForeignKey("dbo.Users", t => t.UserId, cascadeDelete: true)
                .ForeignKey("dbo.Items", t => t.ItemId, cascadeDelete: true)
                .Index(t => t.UserId)
                .Index(t => t.ItemId);
            
            CreateTable(
                "dbo.Users",
                c => new
                    {
                        UserId = c.Int(nullable: false, identity: true),
                        FirstName = c.String(),
                        LastName = c.String(),
                        EmailAddress = c.String(),
                        Status = c.Int(nullable: false),
                        ContactAddress_City = c.String(),
                        ContactAddress_Town = c.String(),
                        ContactAddress_Street = c.String(),
                        ContactAddress_PostalCode = c.String(),
                    })
                .PrimaryKey(t => t.UserId);
            
            CreateTable(
                "dbo.Loans",
                c => new
                    {
                        LoanId = c.Int(nullable: false, identity: true),
                        DateIssued = c.DateTime(),
                        DateReturned = c.DateTime(),
                        DateDueForReturn = c.DateTime(),
                        LoanMethod = c.Int(nullable: false),
                        LoanStatus = c.Int(nullable: false),
                        LoanPeriod = c.Int(),
                        Reservation_ReservationId = c.Int(),
                        User_UserId = c.Int(),
                        ItemCopy_ItemCopyId = c.Int(),
                    })
                .PrimaryKey(t => t.LoanId)
                .ForeignKey("dbo.Reservations", t => t.Reservation_ReservationId)
                .ForeignKey("dbo.Users", t => t.User_UserId)
                .ForeignKey("dbo.ItemCopies", t => t.ItemCopy_ItemCopyId)
                .Index(t => t.Reservation_ReservationId)
                .Index(t => t.User_UserId)
                .Index(t => t.ItemCopy_ItemCopyId);
            
            CreateTable(
                "dbo.Reservations",
                c => new
                    {
                        ReservationId = c.Int(nullable: false, identity: true),
                        Item_ItemId = c.Int(),
                    })
                .PrimaryKey(t => t.ReservationId)
                .ForeignKey("dbo.Items", t => t.Item_ItemId)
                .Index(t => t.Item_ItemId);
            
            CreateTable(
                "dbo.Authors",
                c => new
                    {
                        AuthorId = c.Int(nullable: false, identity: true),
                        FirstName = c.String(),
                        LastName = c.String(),
                        Description = c.String(),
                    })
                .PrimaryKey(t => t.AuthorId);
            
            CreateTable(
                "dbo.Items",
                c => new
                    {
                        ItemId = c.Int(nullable: false, identity: true),
                        PublicationDate = c.DateTime(),
                        Description = c.String(),
                        Title = c.String(),
                        ISBN = c.String(),
                        MagazineName = c.String(),
                        ItemType = c.String(nullable: false, maxLength: 128),
                    })
                .PrimaryKey(t => t.ItemId);
            
            CreateTable(
                "dbo.ItemCategory",
                c => new
                    {
                        ItemId = c.Int(nullable: false),
                        CategoryId = c.Int(nullable: false),
                    })
                .PrimaryKey(t => new { t.ItemId, t.CategoryId })
                .ForeignKey("dbo.Items", t => t.ItemId, cascadeDelete: true)
                .ForeignKey("dbo.Categories", t => t.CategoryId, cascadeDelete: true)
                .Index(t => t.ItemId)
                .Index(t => t.CategoryId);
            
            CreateTable(
                "dbo.AuthorItem",
                c => new
                    {
                        AuthorId = c.Int(nullable: false),
                        ItemId = c.Int(nullable: false),
                    })
                .PrimaryKey(t => new { t.AuthorId, t.ItemId })
                .ForeignKey("dbo.Authors", t => t.AuthorId, cascadeDelete: true)
                .ForeignKey("dbo.Items", t => t.ItemId, cascadeDelete: true)
                .Index(t => t.AuthorId)
                .Index(t => t.ItemId);
            
        }
        
        public override void Down()
        {
            DropIndex("dbo.AuthorItem", new[] { "ItemId" });
            DropIndex("dbo.AuthorItem", new[] { "AuthorId" });
            DropIndex("dbo.ItemCategory", new[] { "CategoryId" });
            DropIndex("dbo.ItemCategory", new[] { "ItemId" });
            DropIndex("dbo.Reservations", new[] { "Item_ItemId" });
            DropIndex("dbo.Loans", new[] { "ItemCopy_ItemCopyId" });
            DropIndex("dbo.Loans", new[] { "User_UserId" });
            DropIndex("dbo.Loans", new[] { "Reservation_ReservationId" });
            DropIndex("dbo.SavedItems", new[] { "ItemId" });
            DropIndex("dbo.SavedItems", new[] { "UserId" });
            DropIndex("dbo.ItemCopies", new[] { "Item_ItemId" });
            DropIndex("dbo.Categories", new[] { "ParentCategory_CategoryId" });
            DropForeignKey("dbo.AuthorItem", "ItemId", "dbo.Items");
            DropForeignKey("dbo.AuthorItem", "AuthorId", "dbo.Authors");
            DropForeignKey("dbo.ItemCategory", "CategoryId", "dbo.Categories");
            DropForeignKey("dbo.ItemCategory", "ItemId", "dbo.Items");
            DropForeignKey("dbo.Reservations", "Item_ItemId", "dbo.Items");
            DropForeignKey("dbo.Loans", "ItemCopy_ItemCopyId", "dbo.ItemCopies");
            DropForeignKey("dbo.Loans", "User_UserId", "dbo.Users");
            DropForeignKey("dbo.Loans", "Reservation_ReservationId", "dbo.Reservations");
            DropForeignKey("dbo.SavedItems", "ItemId", "dbo.Items");
            DropForeignKey("dbo.SavedItems", "UserId", "dbo.Users");
            DropForeignKey("dbo.ItemCopies", "Item_ItemId", "dbo.Items");
            DropForeignKey("dbo.Categories", "ParentCategory_CategoryId", "dbo.Categories");
            DropTable("dbo.AuthorItem");
            DropTable("dbo.ItemCategory");
            DropTable("dbo.Items");
            DropTable("dbo.Authors");
            DropTable("dbo.Reservations");
            DropTable("dbo.Loans");
            DropTable("dbo.Users");
            DropTable("dbo.SavedItems");
            DropTable("dbo.ItemCopies");
            DropTable("dbo.Categories");
        }
    }
}
