﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using NUnit.Framework;
using Library.Tests.Stubs;
using Library.Model;
using Library.Model.Loans;
using Library.Model.Users;
using Library.Model.Items;
using Library.Model.ItemCopies;

namespace Library.Tests
{
	[TestFixture]
	public class LoanTests
	{
		[Test]
		[ExpectedException(typeof(InvalidLoanException))]
		public void A_Loan_Will_Throw_An_Exception_If_It_Is_In_An_Invalid_State()
		{
			Loan loan = new Loan();
			loan.Lend();
		}

		[Test]
		public void A_Loan_Will_Not_Be_Processed_If_User_Status_Is_Not_Active()
		{
			Loan loan = CreateAValidUnprocessedLoan();
			loan.User.Status = Model.Users.Status.Closed;
			loan.Lend();

			Assert.AreEqual(Model.Loans.LoanStatus.Unprocessed, loan.LoanStatus);
		}

		[Test]
		public void A_Loan_Will_Not_Be_Processed_If_Item_Is_Not_Available()
		{
			Loan loan = CreateAValidUnprocessedLoan();
			loan.ItemCopy.IsAvailable = false;
			loan.Lend();

			Assert.AreEqual(Model.Loans.LoanStatus.Unprocessed, loan.LoanStatus);
		}

		[Test]
		public void A_Loan_Status_Will_Change_To_Given_After_Loan_Process()
		{
			Loan loan = CreateAValidUnprocessedLoan();
			loan.Lend();

			Assert.AreEqual(Model.Loans.LoanStatus.Given, loan.LoanStatus);
			Console.WriteLine("User Name:{0}", loan.User.FirstName + "-" + loan.User.LastName);
			Console.WriteLine("Date Issued: {0}", loan.DateIssued.Value.ToShortDateString());
			Console.WriteLine("Due Date For Return: {0}", loan.DateDueForReturn.Value.ToShortDateString());
		}

		[Test]
		public void A_Loan_Status_Will_Change_To_Returned_After_Return_Process()
		{
			Loan loan = CreateAValidUnprocessedLoan();
			loan.Lend();
			
			Console.WriteLine("User Name:{0}", loan.User.FirstName + "-" + loan.User.LastName);
			Console.WriteLine("Date Issued: {0}", loan.DateIssued.Value.ToLongDateString());
			Console.WriteLine("Due Date For Return: {0}", loan.DateDueForReturn.Value.ToLongDateString());

			loan.Return();
			Assert.AreEqual(Model.Loans.LoanStatus.Returned, loan.LoanStatus);
			Console.WriteLine("Date Returned: {0}", loan.DateReturned.Value.ToLongDateString());
		}

		public Loan CreateAValidUnprocessedLoan()
		{
			IAddress address = new StubAddress();
			User user = new User();
			user.Status = Model.Users.Status.Active;
			user.FirstName = "Jhon";
			user.LastName = "Smith";
			user.EmailAddress = "jsmith@live.com";
			user.ContactAddress = new Address("USA", "Vegas", "Wells Fargo", "13006");

			Book book = new Book();
			book.Title = "The C# Programming Language";
			book.Description = "A Must-Read";
			book.ISBN = "1878739735";

			ItemCopy itemCopy = new ItemCopy();
			itemCopy.Item = book;
			itemCopy.IsAvailable = true;
			itemCopy.Tag = "XAX12S";
			itemCopy.Condition = "Used";
			itemCopy.DateOfRecord = DateTime.Now;
			itemCopy.Barcode = "12XS21X2X";

			Loan loan = new Loan();
			loan.User = user;
			loan.ItemCopy = itemCopy;
			loan.LoanPeriod = 2;
			loan.LoanMethod = LoanMethod.TakenOut;
			loan.LoanStatus = Model.Loans.LoanStatus.Unprocessed;
			
			return loan;
		}

	}
}
